const { subtle } = require('node:crypto').webcrypto;
const stringutils = require("./stringutils");

const sharedSecret = process.env.SHARED_SECRET;

const authentication = {
    checkSharedSecret: (providedSecret) => {
        return sharedSecret === providedSecret;
    },
    verifySignature : async (msg, sig, publicKeys) => {
        try {
            for (const hexKey of publicKeys) {
                try {
                    const pubKey = await stringutils.hexToKey(hexKey);
                    const verified = await subtle.verify(
                        'Ed25519',
                        pubKey,
                        sig,
                        msg
                    );
                    if (verified) {
                        return hexKey;
                    }
                } catch (err) {
                    console.log('Failed to verify signature with public key:', hexKey, err);
                }
            }
            return null;
        } catch (err) {
            console.error('Error verifying signature:', err);
        }
    }
};

module.exports = authentication;
