const { subtle } = require('node:crypto').webcrypto;

const stringutils = {
    hexToArray: (hexString) => {
        return Uint8Array.from(Buffer.from(hexString, 'hex'));
    },
    arrayToHex: (hex) => {
        return Buffer.from(hex).toString('hex');
    },
    pemToHex: (pemKey) => {
        const base64 = pemKey.replace(`-----BEGIN PUBLIC KEY-----`, '').replace(`-----END PUBLIC KEY-----`, '').trim().replaceAll('\n','');
        const buffer = Buffer.from(base64, 'base64');
        const hex = buffer.toString('hex');
        return hex
    },
    hexToPem: (hexString) => {
        const buffer = Buffer.from(hexString, 'hex');
        const base64 = buffer.toString('base64');
        // split into 64 character long chunks for PEM format
        const base64Chunks = base64.match(/.{1,64}/g) || [];
        return `-----BEGIN PUBLIC KEY-----\n${base64Chunks.join('\n')}\n-----END PUBLIC KEY-----`;
    },
    hexToKey: async (hexString) => {
        const uint8Array = stringutils.hexToArray(hexString);
        const publicKey = await subtle.importKey(
            "spki",
            uint8Array,
            {
                name: "Ed25519",
            },
            true,
            ["verify"],
        );
        return publicKey;
    },
    pemToKey: async (pemKey) => {
        const base64 = pemKey.replace(`-----BEGIN PUBLIC KEY-----`, '').replace(`-----END PUBLIC KEY-----`, '').trim();
        const buffer = Buffer.from(base64, 'base64');
        const uint8Array = new Uint8Array(buffer);
        const publicKey = await subtle.importKey(
            "spki",
            uint8Array,
            {
                name: "Ed25519",
            },
            true,
            ["verify"],
        );
        return publicKey;
    }
}
module.exports = stringutils;
